@@ -41,8 +41,6 @@ use Cwd 'abs_path';
 # Other environment variable controls:
 # HIP_PATH       : Path to HIP directory, default is one dir level above location of this script.
 # CUDA_PATH      : Path to CUDA SDK (default /usr/local/cuda). Used on NVIDIA platforms only.
-# HSA_PATH       : Path to HSA dir (defaults to ../../hsa relative to abs_path
-#                  of this script). Used on AMD platforms only.
 # HIP_ROCCLR_HOME : Path to HIP/ROCclr directory. Used on AMD platforms only.
 # HIP_CLANG_PATH : Path to HIP-Clang (default to ../../llvm/bin relative to this
 #                  script's abs_path). Used on AMD platforms only.
@@ -52,6 +50,25 @@ if(scalar @ARGV == 0){
     exit(-1);
 }
 
+# retrieve --rocm-path hipcc option from command line.
+# We need to respect this over the env var ROCM_PATH for this compilation.
+sub get_path_options {
+  my $rocm_path="";
+  my $hip_path="";
+  my @CLArgs = @ARGV;
+  foreach $arg (@CLArgs) {
+    if (index($arg,"--rocm-path=") != -1) {
+      ($rocm_path) = $arg=~ /=\s*(.*)\s*$/;
+      next;
+    }
+    if (index($arg,"--hip-path=") != -1) {
+      ($hip_path) = $arg=~ /=\s*(.*)\s*$/;
+      next;
+    }
+  }
+  return ($rocm_path, $hip_path);
+}
+
 $verbose = $ENV{'HIPCC_VERBOSE'} // 0;
 # Verbose: 0x1=commands, 0x2=paths, 0x4=hipcc args
 
@@ -90,11 +107,21 @@ sub delete_temp_dirs {
 my $base_dir;
 BEGIN {
     $base_dir = dirname(Cwd::realpath(__FILE__) );
+    my ($rocm_path, $hip_path) = get_path_options();
+    if ($rocm_path ne '') {
+      # --rocm-path takes precedence over ENV{ROCM_PATH}
+      $ENV{ROCM_PATH}=$rocm_path;
+    }
+    if ($hip_path ne '') {
+      # --rocm-path takes precedence over ENV{ROCM_PATH}
+      $ENV{HIP_PATH}=$hip_path;
+    }
 }
 use lib "$base_dir/";
-use hipvars;
 
+use hipvars;
 $isWindows      =   $hipvars::isWindows;
+$doubleQuote    =   $hipvars::doubleQuote;
 $HIP_RUNTIME    =   $hipvars::HIP_RUNTIME;
 $HIP_PLATFORM   =   $hipvars::HIP_PLATFORM;
 $HIP_COMPILER   =   $hipvars::HIP_COMPILER;
@@ -103,29 +130,17 @@ $CUDA_PATH      =   $hipvars::CUDA_PATH;
 $HIP_PATH       =   $hipvars::HIP_PATH;
 $ROCM_PATH      =   $hipvars::ROCM_PATH;
 $HIP_VERSION    =   $hipvars::HIP_VERSION;
-$HSA_PATH       =   $hipvars::HSA_PATH;
 $HIP_ROCCLR_HOME =   $hipvars::HIP_ROCCLR_HOME;
 
+sub get_normalized_path {
+  return $doubleQuote . $_[0] . $doubleQuote;
+}
+
 if ($HIP_PLATFORM eq "amd") {
-  # If using ROCclr runtime, need to find HIP_ROCCLR_HOME
-  if (!defined $DEVICE_LIB_PATH and -e "$HIP_ROCCLR_HOME/lib/bitcode") {
-    $DEVICE_LIB_PATH = "$HIP_ROCCLR_HOME/lib/bitcode";
-  }
   $HIP_INCLUDE_PATH = "$HIP_ROCCLR_HOME/include";
   if (!defined $HIP_LIB_PATH) {
     $HIP_LIB_PATH = "$HIP_ROCCLR_HOME/lib";
   }
-
-  if (!defined $DEVICE_LIB_PATH) {
-    if (-e "$ROCM_PATH/amdgcn/bitcode") {
-      $DEVICE_LIB_PATH = "$ROCM_PATH/amdgcn/bitcode";
-    }
-    else {
-      # This path is to support an older build of the device library
-      # TODO: To be removed in the future.
-      $DEVICE_LIB_PATH = "$ROCM_PATH/lib";
-    }
-  }
 }
 
 if ($verbose & 0x2) {
@@ -144,30 +159,24 @@ if ($HIP_PLATFORM eq "amd") {
     $execExtension = "";
     if($isWindows) {
         $execExtension = ".exe";
-    } 
-    $HIPCC="$HIP_CLANG_PATH/clang++" . $execExtension;
+    }
+    $HIPCC=get_normalized_path("$HIP_CLANG_PATH/clang++" . $execExtension);
 
     # If $HIPCC clang++ is not compiled, use clang instead
     if ( ! -e $HIPCC ) {
-        $HIPCC="$HIP_CLANG_PATH/clang" . $execExtension;
+        $HIPCC=get_normalized_path("$HIP_CLANG_PATH/clang" . $execExtension);
         $HIPLDFLAGS = "--driver-mode=g++";
     }
     # to avoid using dk linker or MSVC linker
     if($isWindows) {
         $HIPLDFLAGS .= " -fuse-ld=lld";
-        $HIPLDFLAGS .= " --ld-path=$HIP_CLANG_PATH/lld-link.exe";
+        $HIPLDFLAGS .= " --ld-path=" . get_normalized_path("$HIP_CLANG_PATH/lld-link.exe");
     }
-    $HIP_CLANG_VERSION = `$HIPCC --version`;
-    $HIP_CLANG_VERSION=~/.*clang version (\S+).*/;
-    $HIP_CLANG_VERSION=$1;
 
-    # Figure out the target with which llvm is configured
-    $HIP_CLANG_TARGET = `$HIPCC -print-target-triple`;
-    $HIP_CLANG_TARGET = chomp($HIP_CLANG_TARGET);
+    # get Clang RT Builtin path
+    $HIP_CLANG_RT_LIB = `$HIPCC --print-runtime-dir`;
+    chomp($HIP_CLANG_RT_LIB);
 
-    if (! defined $HIP_CLANG_INCLUDE_PATH) {
-        $HIP_CLANG_INCLUDE_PATH = abs_path("$HIP_CLANG_PATH/../lib/clang/$HIP_CLANG_VERSION/include");
-    }
     if (! defined $HIP_INCLUDE_PATH) {
         $HIP_INCLUDE_PATH = "$HIP_PATH/include";
     }
@@ -180,35 +189,16 @@ if ($HIP_PLATFORM eq "amd") {
             print ("HIP_ROCCLR_HOME=$HIP_ROCCLR_HOME\n");
         }
         print ("HIP_CLANG_PATH=$HIP_CLANG_PATH\n");
-        print ("HIP_CLANG_INCLUDE_PATH=$HIP_CLANG_INCLUDE_PATH\n");
         print ("HIP_INCLUDE_PATH=$HIP_INCLUDE_PATH\n");
         print ("HIP_LIB_PATH=$HIP_LIB_PATH\n");
         print ("DEVICE_LIB_PATH=$DEVICE_LIB_PATH\n");
-        print ("HIP_CLANG_TARGET=$HIP_CLANG_TARGET\n");
+        print ("HIP_CLANG_RT_LIB=$HIP_CLANG_RT_LIB\n");
     }
 
-    if ($isWindows) {
-      $HIPCXXFLAGS .= " -std=c++14 -fms-extensions -fms-compatibility";
-    } else {
-      $HIPCXXFLAGS .= " -std=c++11";
-    }
-    $HIPCXXFLAGS .= " -isystem \"$HIP_CLANG_INCLUDE_PATH/..\"";
-    $HIPCFLAGS .= " -isystem \"$HIP_CLANG_INCLUDE_PATH/..\"";
-    $HIPLDFLAGS .= " -L\"$HIP_LIB_PATH\"";
-    if ($isWindows) {
-      $HIPLDFLAGS .= " -lamdhip64";
-    }
     if ($HIP_CLANG_HCC_COMPAT_MODE) {
         ## Allow __fp16 as function parameter and return type.
         $HIPCXXFLAGS .= " -Xclang -fallow-half-arguments-and-returns -D__HIP_HCC_COMPAT_MODE__=1";
     }
-
-    if (not $isWindows) {
-        $HSA_PATH=$ENV{'HSA_PATH'} // "$ROCM_PATH/hsa";
-        $HIPCXXFLAGS .= " -isystem $HSA_PATH/include";
-        $HIPCFLAGS .= " -isystem $HSA_PATH/include";
-    }
-
 } elsif ($HIP_PLATFORM eq "nvidia") {
     $CUDA_PATH=$ENV{'CUDA_PATH'} // '/usr/local/cuda';
     $HIP_INCLUDE_PATH = "$HIP_PATH/include";
@@ -216,22 +206,18 @@ if ($HIP_PLATFORM eq "amd") {
         print ("CUDA_PATH=$CUDA_PATH\n");
     }
 
-    $HIPCC="$CUDA_PATH/bin/nvcc";
+    $HIPCC=get_normalized_path("$CUDA_PATH/bin/nvcc");
     $HIPCXXFLAGS .= " -Wno-deprecated-gpu-targets ";
-    $HIPCXXFLAGS .= " -isystem $CUDA_PATH/include";
-    $HIPCFLAGS .= " -isystem $CUDA_PATH/include";
+    $HIPCXXFLAGS .= " -isystem " . get_normalized_path("$CUDA_PATH/include");
+    $HIPCFLAGS .= " -isystem " . get_normalized_path("$CUDA_PATH/include");
 
-    $HIPLDFLAGS = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L$CUDA_PATH/lib64";
+    $HIPLDFLAGS = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L" . get_normalized_path("$CUDA_PATH/lib64");
 } else {
     printf ("error: unknown HIP_PLATFORM = '$HIP_PLATFORM'");
     printf ("       or HIP_COMPILER = '$HIP_COMPILER'");
     exit (-1);
 }
 
-# Add paths to common HIP includes:
-$HIPCXXFLAGS .= " -isystem \"$HIP_INCLUDE_PATH\"" ;
-$HIPCFLAGS .= " -isystem \"$HIP_INCLUDE_PATH\"" ;
-
 my $compileOnly = 0;
 my $needCXXFLAGS = 0;  # need to add CXX flags to compile step
 my $needCFLAGS = 0;    # need to add C flags to compile step
@@ -240,17 +226,14 @@ my $fileTypeFlag = 0;  # to see if -x flag is mentioned
 my $hasOMPTargets = 0; # If OMP targets is mentioned
 my $hasC = 0;          # options contain a c-style file
 my $hasCXX = 0;        # options contain a cpp-style file (NVCC must force recognition as GPU file)
-my $hasCU = 0;         # options contain a cu-style file (HCC must force recognition as GPU file)
 my $hasHIP = 0;        # options contain a hip-style file (HIP-Clang must pass offloading options)
 my $printHipVersion = 0;    # print HIP version
 my $printCXXFlags = 0;      # print HIPCXXFLAGS
 my $printLDFlags = 0;       # print HIPLDFLAGS
 my $runCmd = 1;
 my $buildDeps = 0;
-my $linkType = 1;
-my $setLinkType = 0;
 my $hsacoVersion = 0;
-my $funcSupp = 0;      # enable function support
+my $funcSupp = 1;      # enable function support
 my $rdc = 0;           # whether -fgpu-rdc is on
 
 my @options = ();
@@ -267,7 +250,12 @@ if($HIP_PLATFORM eq "nvidia"){
     if($ARGV[0] eq "--genco"){
         foreach $isaarg (@ARGV[1..$#ARGV]){
             $ISACMD .= " ";
-            $ISACMD .= $isaarg;
+            # ignore --rocm-path=xxxx on nvcc nvidia platform
+            if ($isaarg !~ /--rocm-path/) {
+              $ISACMD .= $isaarg;
+            } else {
+              print "Ignoring --rocm-path= on nvidia nvcc platform.\n";
+            }
         }
         if ($verbose & 0x1) {
             print "hipcc-cmd: ", $ISACMD, "\n";
@@ -299,6 +287,11 @@ foreach $arg (@ARGV)
     $trimarg =~ s/^\s+|\s+$//g;  # Remive whitespace
     my $swallowArg = 0;
     my $escapeArg = 1;
+    if ($HIP_PLATFORM eq "nvidia") {
+        if (($trimarg =~ /--rocm-path/) or ($trimarg =~ /--hip-path/)) {
+            next;
+        }
+    }
     if ($arg eq '-c' or $arg eq '--genco' or $arg eq '-E') {
         $compileOnly = 1;
         $needLDFLAGS  = 0;
@@ -306,7 +299,7 @@ foreach $arg (@ARGV)
 
     if ($skipOutputFile) {
 	# TODO: handle filename with shell metacharacters
-        $toolArgs .= " \"$arg\"";
+        $toolArgs .= " " . get_normalized_path("$arg");
         $prevArg = $arg;
         $skipOutputFile = 0;
         next;
@@ -326,6 +319,9 @@ foreach $arg (@ARGV)
     # Check target selection option: --offload-arch= and --amdgpu-target=...
     foreach my $targetOpt (@targetOpts) {
         if (substr($arg, 0, length($targetOpt)) eq $targetOpt) {
+             if ($targetOpt eq '--amdgpu-target=') {
+                print "Warning: The --amdgpu-target option has been deprecated and will be removed in the future.  Use --offload-arch instead.\n";
+             }
              # If targets string is not empty, add a comma before adding new target option value.
              $targetsStr .= ($targetsStr ? ',' : '');
              $targetsStr .=  substr($arg, length($targetOpt));
@@ -360,20 +356,11 @@ foreach $arg (@ARGV)
         $compileOnly = 1;
         $buildDeps = 1;
     }
-    if($trimarg eq '-use_fast_math') {
-        $HIPCXXFLAGS .= " -DHIP_FAST_MATH ";
-        $HIPCFLAGS .= " -DHIP_FAST_MATH ";
+    if($trimarg eq '-use-staticlib') {
+        print "Warning: The -use-staticlib option has been deprecated and is no longer needed.\n"
     }
-    if(($trimarg eq '-use-staticlib') and ($setLinkType eq 0))
-    {
-        $linkType = 0;
-        $setLinkType = 1;
-        $swallowArg = 1;
-    }
-    if(($trimarg eq '-use-sharedlib') and ($setLinkType eq 0))
-    {
-        $linkType = 1;
-        $setLinkType = 1;
+    if($trimarg eq '-use-sharedlib') {
+        print "Warning: The -use-sharedlib option has been deprecated and is no longer needed.\n"
     }
     if($arg =~ m/^-O/)
     {
@@ -381,6 +368,7 @@ foreach $arg (@ARGV)
     }
     if($arg =~ '--amdhsa-code-object-version=')
     {
+        print "Warning: The --amdhsa-code-object-version option has been deprecated and will be removed in the future.  Use -mllvm -mcode-object-version instead.\n";
         $arg =~ s/--amdhsa-code-object-version=//;
         $hsacoVersion = $arg;
         $swallowArg = 1;
@@ -395,121 +383,7 @@ foreach $arg (@ARGV)
         $swallowArg = 1;
     }
 
-    ## process linker response file for hip-clang
-    ## extract object files from static library and pass them directly to
-    ## hip-clang in command line.
-    ## ToDo: Remove this after hip-clang switch to lto and lld is able to
-    ## handle clang-offload-bundler bundles.
-    if (($arg =~ m/^-Wl,@/ or $arg =~ m/^@/) and
-        $HIP_PLATFORM eq 'amd') {
-        my @split_arg = (split /\@/, $arg); # arg will have options type(-Wl,@ or @) and filename
-        my $file = $split_arg[1];
-        open my $in, "<:encoding(utf8)", $file or die "$file: $!";
-        my $new_arg = "";
-        my $tmpdir = get_temp_dir ();
-        my $new_file = "$tmpdir/response_file";
-        open my $out, ">", $new_file or die "$new_file: $!";
-        while (my $line = <$in>) {
-            chomp $line;
-            if ($line =~ m/\.a$/ || $line =~ m/\.lo$/) {
-                my $libFile = $line;
-                my $path = abs_path($line);
-                my @objs = split ('\n', `cd $tmpdir; ar xv $path`);
-                ## Check if all files in .a are object files.
-                my $allIsObj = 1;
-                my $realObjs = "";
-                foreach my $obj (@objs) {
-                    chomp $obj;
-                    $obj =~ s/^x - //;
-                    $obj = "$tmpdir/$obj";
-                    my $fileType = `file $obj`;
-                    my $isObj = ($fileType =~ m/ELF/ or $fileType =~ m/COFF/);
-                    $allIsObj = ($allIsObj and $isObj);
-                    if ($isObj) {
-                        $realObjs = ($realObjs . " " . $obj);
-                    } else {
-                        push (@inputs, $obj);
-                        $new_arg = "$new_arg $obj";
-                    }
-                }
-                chomp $realObjs;
-                if ($allIsObj) {
-                    print $out "$line\n";
-                } elsif ($realObjs) {
-                    my($libBaseName, $libDir, $libExt) = fileparse($libFile);
-                    $libBaseName = mktemp($libBaseName . "XXXX") . $libExt;
-                    system("cd $tmpdir; ar rc $libBaseName $realObjs");
-                    print $out "$tmpdir/$libBaseName\n";
-                }
-            } elsif ($line =~ m/\.o$/) {
-                my $fileType = `file $line`;
-                my $isObj = ($fileType =~ m/ELF/ or $fileType =~ m/COFF/);
-                if ($isObj) {
-                    print $out "$line\n";
-                } else {
-                    push (@inputs, $line);
-                    $new_arg = "$new_arg $line";
-                }
-            } else {
-                print $out "$line\n";
-            }
-        }
-        close $in;
-        close $out;
-        $arg = "$new_arg $split_arg[0]\@$new_file";
-        $escapeArg = 0;
-    } elsif (($arg =~ m/\.a$/ || $arg =~ m/\.lo$/) &&
-              $HIP_PLATFORM eq 'amd') {
-        ## process static library for hip-clang
-        ## extract object files from static library and pass them directly to
-        ## hip-clang.
-        ## ToDo: Remove this after hip-clang switch to lto and lld is able to
-        ## handle clang-offload-bundler bundles.
-        my $new_arg = "";
-        my $tmpdir = get_temp_dir ();
-        my $libFile = $arg;
-        my $path = abs_path($arg);
-        my @objs = split ('\n', `cd $tmpdir; ar xv $path`);
-        ## Check if all files in .a are object files.
-        my $allIsObj = 1;
-        my $realObjs = "";
-        foreach my $obj (@objs) {
-            chomp $obj;
-            $obj =~ s/^x - //;
-            $obj = "$tmpdir/$obj";
-            my $fileType = `file $obj`;
-            my $isObj = ($fileType =~ m/ELF/ or $fileType =~ m/COFF/);
-            if ($fileType =~ m/ELF/) {
-                my $sections = `readelf -e -W $obj`;
-                $isObj = !($sections =~ m/__CLANG_OFFLOAD_BUNDLE__/);
-            }
-            $allIsObj = ($allIsObj and $isObj);
-            if ($isObj) {
-                $realObjs = ($realObjs . " " . $obj);
-            } else {
-                push (@inputs, $obj);
-                if ($new_arg ne "") {
-                    $new_arg .= " ";
-                }
-                $new_arg .= "$obj";
-            }
-        }
-        chomp $realObjs;
-        if ($allIsObj) {
-            $new_arg = $arg;
-        } elsif ($realObjs) {
-            my($libBaseName, $libDir, $libExt) = fileparse($libFile);
-            $libBaseName = mktemp($libBaseName . "XXXX") . $libExt;
-            system("cd $tmpdir; ar rc $libBaseName $realObjs");
-            $new_arg .= " $tmpdir/$libBaseName";
-        }
-        $arg = "$new_arg";
-        $escapeArg = 0;
-        if ($toolArgs =~ m/-Xlinker$/) {
-            $toolArgs = substr $toolArgs, 0, -8;
-            chomp $toolArgs;
-        }
-    } elsif ($arg eq '-x') {
+    if ($arg eq '-x') {
         $fileTypeFlag = 1;
     } elsif (($arg eq 'c' and $prevArg eq '-x') or ($arg eq '-xc')) {
         $fileTypeFlag = 1;
@@ -539,12 +413,11 @@ foreach $arg (@ARGV)
         # Process HIPCC options here:
         if ($arg =~ m/^--hipcc/) {
             $swallowArg = 1;
-            #if $arg eq "--hipcc_profile") {  # Example argument here, hipcc
-            #
-            #}
             if ($arg eq "--hipcc-func-supp") {
+              print "Warning: The --hipcc-func-supp option has been deprecated and will be removed in the future.\n";
               $funcSupp = 1;
             } elsif ($arg eq "--hipcc-no-func-supp") {
+              print "Warning: The --hipcc-no-func-supp option has been deprecated and will be removed in the future.\n";
               $funcSupp = 0;
             }
         } else {
@@ -568,6 +441,9 @@ foreach $arg (@ARGV)
                 $needCXXFLAGS = 1;
                 if ($HIP_COMPILE_CXX_AS_HIP eq '0' or $HIP_PLATFORM ne "amd" or $hasOMPTargets eq 1) {
                     $hasCXX = 1;
+                    if ($HIP_PLATFORM eq "nvidia") {
+                        $toolArgs .= " -x cu";
+                    }
                 } elsif ($HIP_PLATFORM eq "amd") {
                     $hasHIP = 1;
                     $toolArgs .= " -x hip";
@@ -577,8 +453,8 @@ foreach $arg (@ARGV)
                 if ($HIP_PLATFORM eq "amd") {
                     $hasHIP = 1;
                     $toolArgs .= " -x hip";
-                } else {
-                    $hasCU = 1;
+                } elsif ($HIP_PLATFORM eq "nvidia") {
+                    $toolArgs .= " -x cu";
                 }
             }
         }
@@ -598,9 +474,6 @@ foreach $arg (@ARGV)
     if (not $isWindows and $escapeArg) {
         $arg =~ s/[^-a-zA-Z0-9_=+,.\/]/\\$&/g;
     }
-    if ($isWindows and $escapeArg) {
-        $arg =~ s/[^-a-zA-Z0-9_=+,.:\/\\]/\\$&/g;
-    }
     $toolArgs .= " $arg" unless $swallowArg;
     $prevArg = $arg;
 }
@@ -662,10 +535,6 @@ if($HIP_PLATFORM eq "amd"){
     $ENV{HCC_EXTRA_LIBRARIES}="\n";
 }
 
-if ($hasCXX and $HIP_PLATFORM eq 'nvidia') {
-    $HIPCXXFLAGS .= " -x cu";
-}
-
 if ($buildDeps and $HIP_PLATFORM eq 'nvidia') {
     $HIPCXXFLAGS .= " -M -D__CUDACC__";
     $HIPCFLAGS .= " -M -D__CUDACC__";
@@ -675,12 +544,6 @@ if ($buildDeps and $HIP_PLATFORM eq 'amd') {
     $HIPCXXFLAGS .= " --cuda-host-only";
 }
 
-# Add --hip-link only if it is compile only and -fgpu-rdc is on.
-if ($rdc and !$compileOnly and $HIP_PLATFORM eq 'amd') {
-    $HIPLDFLAGS .= " --hip-link";
-    $HIPLDFLAGS .= $HIPLDARCHFLAGS;
-}
-
 # hipcc currrently requires separate compilation of source files, ie it is not possible to pass
 # CPP files combined with .O files
 # Reason is that NVCC uses the file extension to determine whether to compile in CUDA mode or
@@ -700,28 +563,28 @@ if ($HIP_PLATFORM eq "amd") {
         }
     }
 
-    if ($hasHIP) {
-        if ($DEVICE_LIB_PATH ne "$ROCM_PATH/amdgcn/bitcode") {
-            $HIPCXXFLAGS .= " --hip-device-lib-path=\"$DEVICE_LIB_PATH\"";
-        }
-    }
-    if (not $isWindows) {
-        $HIPLDFLAGS .= " -lgcc_s -lgcc -lpthread -lm -lrt";
+    # If the HIP_PATH env var is defined, pass that path to Clang
+    if ($ENV{'HIP_PATH'}) {
+        my $hip_path_flag = " --hip-path=" . get_normalized_path("$HIP_PATH");
+        $HIPCXXFLAGS .= $hip_path_flag;
+        $HIPLDFLAGS .= $hip_path_flag;
     }
 
-    if (not $isWindows  and not $compileOnly) {
-      if ($linkType eq 0) {
-        $toolArgs = " -L$HIP_LIB_PATH -lamdhip64 -L$ROCM_PATH/lib -lhsa-runtime64 -ldl -lnuma " . ${toolArgs};
-      } else {
-        $toolArgs = ${toolArgs} . " -Wl,--enable-new-dtags -Wl,-rpath=$HIP_LIB_PATH:$ROCM_PATH/lib -lamdhip64 ";
-      }
-      # To support __fp16 and _Float16, explicitly link with compiler-rt
-      $HIP_CLANG_BUILTIN_LIB="$HIP_CLANG_PATH/../lib/clang/$HIP_CLANG_VERSION/lib/$HIP_CLANG_TARGET/libclang_rt.builtins.a";
-      if (-e $HIP_CLANG_BUILTIN_LIB) {
-        $toolArgs .= " -L$HIP_CLANG_PATH/../lib/clang/$HIP_CLANG_VERSION/lib/$HIP_CLANG_TARGET -lclang_rt.builtins "
-      } else {
-        $toolArgs .= " -L$HIP_CLANG_PATH/../lib/clang/$HIP_CLANG_VERSION/lib/linux -lclang_rt.builtins-x86_64 "
-      }
+    if ($hasHIP) {
+        if (defined $DEVICE_LIB_PATH) {
+            $HIPCXXFLAGS .= " --hip-device-lib-path=" . get_normalized_path("$DEVICE_LIB_PATH");
+        }
+    }
+
+    if (!$compileOnly) {
+        $HIPLDFLAGS .= " --hip-link";
+        if ($rdc) {
+            $HIPLDFLAGS .= $HIPLDARCHFLAGS;
+        }
+        if (not $isWindows) {
+            $HIPLDFLAGS .= " --rtlib=compiler-rt -unwindlib=libgcc";
+
+        }
     }
 }
 
